#include <ultra64.h>
#include "thread.h"
#include "textlib.h"
#include "cont.h"

#define uchar unsigned char
#define ulong unsigned long
#define uint unsigned short
#define box_w 160
#define box_h 100
#define pi 3.1415926L
#define max(a,b) ((a) > (b) ? (a) : (b))
#define NUM_MESSAGE 2

extern u16     *bitmap_buf;
extern u16      bitmap_buf1[];
extern u16      bitmap_buf2[];
extern u8       gheightmap[];
extern u16      gcolormap[];
extern u16      button;
extern int      GetButton (void);
extern OSMesgQueue retraceMessageQ;
extern OSMesgQueue pifMesgQueue;
extern OSMesg   retraceMessageBuf;
extern OSMesg   dummyMessage;
extern OSMesg   pifMesgBuf[NUM_MESSAGE];


uchar           body[box_w * box_h];	// buffer for the bitmap body
u16             ColourTab[256];
uchar           tab1[64000];	// table one for the plasma
uchar           tab2[64000];	// table two for the plasma
int             bank = 0;

void
put_pixel (int x, int y, int c)
{
    u16            *scr;

    scr = bitmap_buf + x + (y * 320);
    *scr = c;
}

void
CalcTab1 ()         
{
    long            i = 0,
                    j = 0;

    while (i < box_h * 2)
    {
	j = 0;
    while (j < box_w * 2)
	{
	    tab1[(i * box_w * 2) + j] = (uchar) ((sqrtf (16.0 + (box_h - i) * (box_h - i) + (box_w - j) * (box_w - j)) - 4) * 5);
	    j++;
	}
	i++;
    }
}
void
CalcTab2 ()         
{
    long            i = 0,
                    j = 0;
    float           temp;

    while (i < box_h * 2)
    {
	j = 0;
	while (j < box_w * 2)
	{
	    temp = sqrtf (16.0 + (box_h - i) * (box_h - i) + (box_w - j) * (box_w - j)) - 4;
	    tab2[(i * box_w * 2) + j] = (sinf (temp / 9.5) + 1) * 90;
	    j++;
	}
	i++;
    }
}

void
CalculateColors (void)
{
#define icecol(a) ((unsigned int)a & 31)

    int             i = 0,
                    rc = 0,
                    bc = 0,
                    gc = 0;

    i = 0;
    while (i < 32)
    {
	ColourTab[i] = (icecol (rc) << 11) + (icecol (gc) << 6) + (icecol (bc) << 1) + 1;
	i++;
	rc++;
    }

    rc = 31;
    bc = 0;
    gc = 0;
    i = 32;
    while (i < 64)
    {
	ColourTab[i] = (icecol (rc) << 11) + (icecol (gc) << 6) + (icecol (bc) << 1) + 1;
	i++;
	gc++;
    }

    rc = 31;
    bc = 0;
    gc = 31;
    i = 64;
    while (i < 96)
    {
	ColourTab[i] = (icecol (rc) << 11) + (icecol (gc) << 6) + (icecol (bc) << 1) + 1;
	i++;
	bc++;
    }

    rc = 31;
    bc = 31;
    gc = 31;
    i = 96;
    while (i < 128)
    {
	ColourTab[i] = (icecol (rc) << 11) + (icecol (gc) << 6) + (icecol (bc) << 1) + 1;
	i++;
	bc--;
    }

    rc = 31;
    bc = 0;
    gc = 31;
    i = 128;
    while (i < 160)
    {
	ColourTab[i] = (icecol (rc) << 11) + (icecol (gc) << 6) + (icecol (bc) << 1) + 1;
	i++;
	gc--;
    }

    rc = 31;
    bc = 0;
    gc = 0;
    i = 160;
    while (i < 192)
    {
	ColourTab[i] = (icecol (rc) << 11) + (icecol (gc) << 6) + (icecol (bc) << 1) + 1;
	i++;
	rc--;
    }

    rc = 0;
    bc = 0;
    gc = 0;
    i = 192;
    while (i < 224)
    {
	ColourTab[i] = (icecol (rc) << 11) + (icecol (gc) << 6) + (icecol (bc) << 1) + 1;
	i++;
	rc++;
	bc++;
    }

    rc = 31;
    bc = 31;
    gc = 0;
    i = 224;
    while (i < 256)
    {
	ColourTab[i] = (icecol (rc) << 11) + (icecol (gc) << 6) + (icecol (bc) << 1) + 1;
	i++;
	rc--;
	bc--;
    }
}

void
CalculateBody (long x1, long y1, long x2, long y2, long x3, long y3, long x4, long y4, long roll)
{
    long            i = 0,
                    j = 0,
                    k = 0;

    while (i < box_h)
    {
	j = 0;
	k += box_w;
	while (j < box_w)
	{
	    // this is the heart of the plasma
	    body[k + j] =
		tab1[320 * (i + y1) + j + x1] + roll +
		tab2[320 * (i + y2) + j + x2] +
		tab2[320 * (i + y3) + j + x3] +
		tab2[320 * (i + y4) + j + x4]
		;
	    j++;
	}
	i++;
    }
}
void
DisplayObject (int x, int y)
{
    u16             x1,
                    y1;

// wait for 2 vsync's to complete
    for (x1 = 0; x1 < 2; x1++)
    {
	if (MQ_IS_FULL (&retraceMessageQ))
	    (void) osRecvMesg (&retraceMessageQ, &dummyMessage, OS_MESG_BLOCK);
	(void) osRecvMesg (&retraceMessageQ, &dummyMessage, OS_MESG_BLOCK);
    }

    for (y1 = 0; y1 < 100; y1++)
    {
	for (x1 = 0; x1 < 160; x1++)
	{
	    put_pixel (x1 + 80, y1 + 50, ColourTab[body[(y1 * 160) + x1]]);
	}
    }

}


#define	KEY_PLUS  1
#define	KEY_MINUS 2
#define	KEY_UP	  3
#define	KEY_DOWN  4
#define	KEY_LEFT  5
#define	KEY_RIGHT 6
#define KEY_START 7
int
keyTest2 (u16 kc)
{
    button = 0;
    GetButton ();

    if ((kc == KEY_START) && (button == START_BUTTON))
	return (-1);

    if ((kc == KEY_PLUS) && (button == CONT_B))
	return (1);

    if ((kc == KEY_MINUS) && (button == CONT_A))
	return (1);

    if ((kc == KEY_UP) && (button == CONT_UP))
	return (1);

    if ((kc == KEY_DOWN) && (button == CONT_DOWN))
	return (1);

    if ((kc == KEY_LEFT) && (button == CONT_LEFT))
	return (1);

    if ((kc == KEY_RIGHT) && (button == CONT_RIGHT))
	return (1);

    return (0);
}

int
CheckKeys2 ()
{
    if (keyTest2 (KEY_START))
    {
	return (-1);
    }

    return 0;
}


void
plasma_main (void)
{
    float           circle1 = 0,
                    circle2 = 0,
                    circle3 = 0,
                    circle4 = 0,
                    circle5 = 0,
                    circle6 = 0,
                    circle7 = 0,
                    circle8 = 0;
    int             x1,
                    y1,
                    x2,
                    y2,
                    x3,
                    y3,
                    x4,
                    y4,
                    roll = 0,
                    i = 0;


    CalcTab1 ();
    CalcTab2 ();

    set_xy (0, 10);
    setcolor (255, 255, 255);
    draw_center ("Plasma Example by Frac/BB", 1);

    CalculateColors ();
    while (CheckKeys2 () != -1)
    {
	i++;
	circle1 += 0.085 / 6;
	circle2 -= 0.1 / 6;
	circle3 += .3 / 6;
	circle4 -= .2 / 6;
	circle5 += .4 / 6;
	circle6 -= .15 / 6;
	circle7 += .35 / 6;
	circle8 -= .05 / 6;
	x2 = (box_w / 2) + (box_w / 2) * sinf (circle1);
	y2 = (box_h / 2) + (box_h / 2) * cosf (circle2);
	x1 = (box_w / 2) + (box_w / 2) * cosf (circle3);
	y1 = (box_h / 2) + (box_h / 2) * sinf (circle4);
	x3 = (box_w / 2) + (box_w / 2) * cosf (circle5);
	y3 = (box_h / 2) + (box_h / 2) * sinf (circle6);
	x4 = (box_w / 2) + (box_w / 2) * cosf (circle7);
	y4 = (box_h / 2) + (box_h / 2) * sinf (circle8);

	CalculateBody (x1, y1, x2, y2, x3, y3, x4, y4, roll += 5);
	DisplayObject (0, 0);
    }
}
